==> msee-0.2.1.tgz: package/lib/msee.js <==
var fs = require('fs');
var path = require('path');
var marked = require('marked');
var cardinal = require('cardinal');
var xtend = require('xtend');
var color = require('./color');
var table = require('text-table');

var defaultOptions = {
    collapseNewlines: true,
    space: '',
    hrStart: '',
    hrChar: '-',
    hrEnd: '',
    headingStart: '\n',
    headingEnd: '\n\n',
    headingIndentChar: '#',
    codeStart: '\n',
    codeEnd: '\n\n',
    codePad: '    ',
    blockquoteStart: '\n',
    blockquoteEnd: '\n\n',
    blockquoteColor: 'blockquote',
    blockquotePad: '  > ',
    blockquotePadColor: 'syntax',
    listStart: '\n',
    listEnd: '\n',
    listItemStart: '',
    listItemEnd: '\n',
    listItemColor: 'ul',
    listItemPad: '  * ',
    listItemPadColor: 'syntax',
    paragraphStart: '',
    paragraphEnd: '',
    paragraphColor: 'paragraph',
    paragraphPad: '',
    paragraphPadColor: 'paragraph',
    width: process.stdout.columns ? process.stdout.columns - 2 : 84,
    tableStart: '\n',
    tableSeparator: ' ',
    tableEnd: '\n\n'
};

var tokens;
var inline;
var token;
var blockDepth = 0;

function processInline(src, options) {
    var out = '';
    var cap;

    function outLink (title, href) {
        out += '[' + color(title, 'strong') + '](' + color(href, 'link') + ')';
    }

    while (src) {
        // escape
        if (cap = inline.rules.escape.exec(src)) {
          src = src.substring(cap[0].length);
          out += cap[1];
          continue;
        }

        // code
        if (cap = inline.rules.code.exec(src)) {
            src = src.substring(cap[0].length);
            out += color(cap[2], 'code');
            continue;
        }

        // autolink
        if (cap = inline.rules.autolink.exec(src)) {
            src = src.substring(cap[0].length);
            out += color(cap[0], 'link');
            continue;
        }

        // url (gfm)
        if (cap = inline.rules.url.exec(src)) {
            src = src.substring(cap[0].length);
            outLink(cap[1], cap[1]);
          continue;
        }

        // tag
        if (cap = inline.rules.tag.exec(src)) {
            src = src.substring(cap[0].length);
            out += cap[0];
            continue;
        }

        // link
        if (cap = inline.rules.link.exec(src)) {
            src = src.substring(cap[0].length);
            outLink(cap[1], cap[2]);
            continue;
        }

        // reflink, nolink
        if ((cap = inline.rules.reflink.exec(src))
                || (cap = inline.rules.nolink.exec(src))) {
            src = src.substring(cap[0].length);
            out += cap[0];
            continue;
        }

        // strong
        if (cap = inline.rules.strong.exec(src)) {
            src = src.substring(cap[0].length);
            out += color(processInline(cap[2] || cap[1]), 'strong');
            continue;
        }

        // em
        if (cap = inline.rules.em.exec(src)) {
            src = src.substring(cap[0].length);
            out += color(processInline(cap[2] || cap[1]), 'em');
            continue;
        }

        // br
        if (cap = inline.rules.br.exec(src)) {
            src = src.substring(cap[0].length);
            out += '\n';
            continue;
        }

        // del (gfm)
        if (cap = inline.rules.del.exec(src)) {
            src = src.substring(cap[0].length);
            out += color(processInline(cap[1]), 'del');
            continue;
        }

        // text
        if (cap = inline.rules.text.exec(src)) {
            src = src.substring(cap[0].length);
            out += cap[0];
            continue;
        }

        if (src) {
          throw new Error('Infinite loop on byte: ' + src.charCodeAt(0));
        }
    }

    return out;
}

function processToken(options) {
    var type = token.type;
    var text = token.text;
    var content;

    switch (type) {
        case 'space': {
            return options.space;
        }
        case 'hr': {
            var hrStr = new Array(options.width).join(options.hrChar) + '\n';
            return options.hrStart + color(hrStr, type) + options.hrEnd;
        }
        case 'heading': {
            var syntaxFlag = color(
                Array(token.depth + 1).join(options.headingIndentChar),
                'syntax'
            );
            text = processInline(text);
            content = color(text, type);

            return options.headingStart + syntaxFlag + ' ' + content + options.headingEnd;
        }
        case 'code': {
            content = '';

            try {
                content = cardinal.highlight(text);
            }
            catch (e) {
                content = color(text, type);
            }

            content = blockFormat(content, {
                pad_str: options.codePad.lang ? (typeof options.codePad.lang[token.lang] === 'string' ? options.codePad.lang[token.lang] : options.defaultCodePad) : options.codePad,
                width: options.width
            });

            return options.codeStart + content + options.codeEnd;
        }
        case 'table': {
            content = tableFormat(token, options);
            return options.tableStart + content + options.tableEnd;
        }
        case 'blockquote_start': {
            content = '';
            blockDepth++;

            while (next().type !== 'blockquote_end') {
                content += processToken(options);
            }
            content = blockFormat(content, {
                block_color: options.blockquoteColor,
                pad_str: options.blockquotePad,
                pad_color: options.blockquotePadColor,
                width: options.width
            });

            blockDepth--;
            return options.blockquoteStart + content + options.blockquoteEnd;
        }
        case 'list_start': {
            content = '';

            while (next().type !== 'list_end') {
                content += processToken(options);
            }

            return options.listStart + content + options.listEnd;
        }
        case 'list_item_start': {
            content = '';

            while (next().type !== 'list_item_end') {
                if (type === 'text') {
                    content += text;
                } else {
                    content += processToken(options);
                }
            }
            content = blockFormat(processInline(content), {
                block_color: options.listItemColor,
                first_pad_str: options.listItemPad,
                pad_str: new Array(options.listItemPad.length + 1).join(' '),
                pad_color: options.listItemPadColor,
                width: options.width
            });
            return options.listItemStart + content + options.listItemEnd;
        }
        case 'paragraph': {
            if (blockDepth > 0) {
                process.exit()
                return text.split('\n').join(' ');
            }
            content = blockFormat(processInline(text), {
                block_color: options.paragraphColor,
                pad_str: options.paragraphPad,
                pad_color: options.paragraphPadColor,
                width: options.width
            });
            return options.paragraphStart + content + options.paragraphEnd;
        }
        default: {
            if (text) {
                return text;
            }
        }
    }
}

function next() {
    return token = tokens.shift();
}

function blockFormat(src, opts) {
    var lines = src.split('\n');
    var padStr = opts.pad_str || '';
    var firstPadStr = opts.first_pad_str || padStr;
    var padColor = opts.pad_color || opts.block_color;
    var retLines = [];
    var secondWidth = opts.width - padStr.length;
    var firstWidth = opts.width - firstPadStr.length;
    var first = true;

    if (padColor) {
        padStr = color(padStr, padColor);
        firstPadStr = color(firstPadStr, padColor);
    }

    lines.forEach(function(rawLine) {
        for (var i = 0; i < rawLine.length;) {
            var width = first ? firstWidth : secondWidth;
            var end = i + width;
            var line;
            if (end > rawLine.length) {
                line = rawLine.substr(i);
                i = end;
            } else {
                var lastSpace = rawLine.lastIndexOf(' ', end);
                if (lastSpace === -1) {
                    line = rawLine.substring(i, end);
                    i = end;
                } else {
                    line = rawLine.substring(i, lastSpace);
                    i = lastSpace + 1;
                }   
            }
            if (opts.block_color) {
                line = color(line, opts.block_color);
            }
            retLines.push((first ? firstPadStr : padStr) + line);
            first = false;
        }
    });

    return retLines.join('\n');
}

function tableFormat (token, options) {
    var aligns = token.align.map(function(a){ return (a===null)? 'l' : a[0] });
    var rows = token.cells.map(function(row){ return row.map(processInline) });
    rows.unshift(token.header.map(function(){ return '--' }));
    rows.unshift(token.header.map(function(s){ return processInline('**'+s+'**') }));
    return table(rows, {
        align: aligns,
        stringLength: ansiLength,
        hsep: options.tableSeparator
    });
}

/**
 * Trim formatting from string
 * @see https://github.com/medikoo/cli-color/blob/master/trim.js
 */

function ansiLength (str) {
    var r = new RegExp('\x1b(?:\\[(?:\\d+[ABCDEFGJKSTm]|\\d+;\\d+[Hfm]|' +
    '\\d+;\\d+;\\d+m|6n|s|u|\\?25[lh])|\\w)', 'g');
    return str.replace(r, '').length;
}

exports.parse = function(text, options) {
    tokens = marked.lexer(text);
    inline = new marked.InlineLexer(tokens.links);
    options = xtend(defaultOptions, options);

    var outputArr = [];
    var output;

    while (next()) {
        outputArr.push(processToken(options));
    }

    output = outputArr.join('\n')

    if (options.collapseNewlines) {
        output = output.replace(/\n\n/g, '\n');
    }

    tokens = null;
    token = null;

    return output;
}

exports.parseFile = function(file, options) {
    var filePath = path.resolve(__dirname, file);
    var ret = '';

    try {
        var text = fs.readFileSync(filePath).toString();
        ret = exports.parse(text, options);
    }
    catch (e) {
        throw e;
    }

    return ret;
}

==> msee-0.2.2.tgz: package/lib/msee.js <==
var fs = require('fs');
var path = require('path');
var marked = require('marked');
var cardinal = require('cardinal');
var xtend = require('xtend');
var color = require('./color');
var table = require('text-table');
var vw = require('@leichtgewicht/visualwidth');

var defaultOptions = {
    collapseNewlines: true,
    space: '',
    hrStart: '',
    hrChar: '-',
    hrEnd: '',
    headingStart: '\n',
    headingEnd: '\n\n',
    headingIndentChar: '#',
    codeStart: '\n',
    codeEnd: '\n\n',
    codePad: '    ',
    blockquoteStart: '\n',
    blockquoteEnd: '\n\n',
    blockquoteColor: 'blockquote',
    blockquotePad: '  > ',
    blockquotePadColor: 'syntax',
    listStart: '\n',
    listEnd: '\n',
    listItemStart: '',
    listItemEnd: '\n',
    listItemColor: 'ul',
    listItemPad: '  * ',
    listItemPadColor: 'syntax',
    paragraphStart: '',
    paragraphEnd: '',
    paragraphColor: 'paragraph',
    paragraphPad: '',
    paragraphPadColor: 'paragraph',
    width: process.stdout.columns ? process.stdout.columns - 2 : 84,
    tableStart: '\n',
    tableSeparator: ' ',
    tableEnd: '\n\n'
};

var tokens;
var inline;
var token;
var blockDepth = 0;

function processInline(src, options) {
    var out = '';
    var cap;

    function outLink (title, href) {
        out += '[' + color(title, 'strong') + '](' + color(href, 'link') + ')';
    }

    while (src) {
        // escape
        if (cap = inline.rules.escape.exec(src)) {
          src = src.substring(cap[0].length);
          out += cap[1];
          continue;
        }

        // code
        if (cap = inline.rules.code.exec(src)) {
            src = src.substring(cap[0].length);
            out += color(cap[2], 'code');
            continue;
        }

        // autolink
        if (cap = inline.rules.autolink.exec(src)) {
            src = src.substring(cap[0].length);
            out += color(cap[0], 'link');
            continue;
        }

        // url (gfm)
        if (cap = inline.rules.url.exec(src)) {
            src = src.substring(cap[0].length);
            outLink(cap[1], cap[1]);
          continue;
        }

        // tag
        if (cap = inline.rules.tag.exec(src)) {
            src = src.substring(cap[0].length);
            out += cap[0];
            continue;
        }

        // link
        if (cap = inline.rules.link.exec(src)) {
            src = src.substring(cap[0].length);
            outLink(cap[1], cap[2]);
            continue;
        }

        // reflink, nolink
        if ((cap = inline.rules.reflink.exec(src))
                || (cap = inline.rules.nolink.exec(src))) {
            src = src.substring(cap[0].length);
            out += cap[0];
            continue;
        }

        // strong
        if (cap = inline.rules.strong.exec(src)) {
            src = src.substring(cap[0].length);
            out += color(processInline(cap[2] || cap[1]), 'strong');
            continue;
        }

        // em
        if (cap = inline.rules.em.exec(src)) {
            src = src.substring(cap[0].length);
            out += color(processInline(cap[2] || cap[1]), 'em');
            continue;
        }

        // br
        if (cap = inline.rules.br.exec(src)) {
            src = src.substring(cap[0].length);
            out += '\n';
            continue;
        }

        // del (gfm)
        if (cap = inline.rules.del.exec(src)) {
            src = src.substring(cap[0].length);
            out += color(processInline(cap[1]), 'del');
            continue;
        }

        // text
        if (cap = inline.rules.text.exec(src)) {
            src = src.substring(cap[0].length);
            out += cap[0];
            continue;
        }

        if (src) {
          throw new Error('Infinite loop on byte: ' + src.charCodeAt(0));
        }
    }

    return out;
}

function processToken(options) {
    var type = token.type;
    var text = token.text;
    var content;

    switch (type) {
        case 'space': {
            return options.space;
        }
        case 'hr': {
            var hrStr = new Array(options.width).join(options.hrChar) + '\n';
            return options.hrStart + color(hrStr, type) + options.hrEnd;
        }
        case 'heading': {
            var syntaxFlag = color(
                Array(token.depth + 1).join(options.headingIndentChar),
                'syntax'
            );
            text = processInline(text);
            content = color(text, type);

            return options.headingStart + syntaxFlag + ' ' + content + options.headingEnd;
        }
        case 'code': {
            content = '';

            try {
                content = cardinal.highlight(text);
            }
            catch (e) {
                content = color(text, type);
            }

            content = blockFormat(content, {
                pad_str: options.codePad.lang ? (typeof options.codePad.lang[token.lang] === 'string' ? options.codePad.lang[token.lang] : options.defaultCodePad) : options.codePad,
                width: options.width
            });

            return options.codeStart + content + options.codeEnd;
        }
        case 'table': {
            content = tableFormat(token, options);
            return options.tableStart + content + options.tableEnd;
        }
        case 'blockquote_start': {
            content = '';
            blockDepth++;

            while (next().type !== 'blockquote_end') {
                content += processToken(options);
            }
            content = blockFormat(content, {
                block_color: options.blockquoteColor,
                pad_str: options.blockquotePad,
                pad_color: options.blockquotePadColor,
                width: options.width
            });

            blockDepth--;
            return options.blockquoteStart + content + options.blockquoteEnd;
        }
        case 'list_start': {
            content = '';

            while (next().type !== 'list_end') {
                content += processToken(options);
            }

            return options.listStart + content + options.listEnd;
        }
        case 'list_item_start': {
            content = '';

            while (next().type !== 'list_item_end') {
                if (type === 'text') {
                    content += text;
                } else {
                    content += processToken(options);
                }
            }
            content = blockFormat(processInline(content), {
                block_color: options.listItemColor,
                first_pad_str: options.listItemPad,
                pad_str: new Array(options.listItemPad.length + 1).join(' '),
                pad_color: options.listItemPadColor,
                width: options.width
            });
            return options.listItemStart + content + options.listItemEnd;
        }
        case 'paragraph': {
            if (blockDepth > 0) {
                process.exit()
                return text.split('\n').join(' ');
            }
            content = blockFormat(processInline(text), {
                block_color: options.paragraphColor,
                pad_str: options.paragraphPad,
                pad_color: options.paragraphPadColor,
                width: options.width
            });
            return options.paragraphStart + content + options.paragraphEnd;
        }
        default: {
            if (text) {
                return text;
            }
        }
    }
}

function next() {
    return token = tokens.shift();
}

function blockFormat(src, opts) {
    var lines = src.split('\n');
    var padStr = opts.pad_str || '';
    var firstPadStr = opts.first_pad_str || padStr;
    var padColor = opts.pad_color || opts.block_color;
    var retLines = [];
    var secondWidth = opts.width - vw.width(padStr, true);
    var firstWidth = opts.width - vw.width(firstPadStr, true);
    var first = true;

    if (padColor) {
        padStr = color(padStr, padColor);
        firstPadStr = color(firstPadStr, padColor);
    }

    lines.forEach(function(rawLine) {
        var rawWidth = vw.width(rawLine, true);
        for (var i = 0; i < rawWidth;) {
            var width = first ? firstWidth : secondWidth;
            var end = i + width;
            var line;
            if (end > rawWidth) {
                line = vw.substr(rawLine, i, null, true);
                i = end;
            } else {
                var lastSpace = vw.lastIndexOf(rawLine, ' ', end, true);
                if (lastSpace === -1 || lastSpace < i) {
                    line = vw.substring(rawLine, i, end, true);
                    i = end;
                } else {
                    line = vw.substring(rawLine, i, lastSpace, true);
                    i = lastSpace+1;
                }   
            }
            if (opts.block_color) {
                line = color(line, opts.block_color);
            }
            retLines.push((first ? firstPadStr : padStr) + line);
            first = false;
        }
    });

    return retLines.join('\n');
}

function tableFormat (token, options) {
    var aligns = token.align.map(function(a){ return (a===null)? 'l' : a[0] });
    var rows = token.cells.map(function(row){ return row.map(processInline) });
    rows.unshift(token.header.map(function(){ return '--' }));
    rows.unshift(token.header.map(function(s){ return processInline('**'+s+'**') }));
    return table(rows, {
        align: aligns,
        stringLength: ansiLength,
        hsep: options.tableSeparator
    });
}

/**
 * Trim formatting from string
 * @see https://github.com/medikoo/cli-color/blob/master/trim.js
 */

function ansiLength (str) {
    var r = new RegExp('\x1b(?:\\[(?:\\d+[ABCDEFGJKSTm]|\\d+;\\d+[Hfm]|' +
    '\\d+;\\d+;\\d+m|6n|s|u|\\?25[lh])|\\w)', 'g');
    return str.replace(r, '').length;
}

exports.parse = function(text, options) {
    tokens = marked.lexer(text);
    inline = new marked.InlineLexer(tokens.links);
    options = xtend(defaultOptions, options);

    var outputArr = [];
    var output;

    while (next()) {
        outputArr.push(processToken(options));
    }

    output = outputArr.join('\n')

    if (options.collapseNewlines) {
        output = output.replace(/\n\n/g, '\n');
    }

    tokens = null;
    token = null;

    return output;
}

exports.parseFile = function(file, options) {
    var filePath = path.resolve(__dirname, file);
    var ret = '';

    try {
        var text = fs.readFileSync(filePath).toString();
        ret = exports.parse(text, options);
    }
    catch (e) {
        throw e;
    }

    return ret;
}
